//
//  MyView.m
//  TestLines
//
//  Created by Cameron Hayne on 27/02/06.
//  Copyright 2006 Hayne of Tintagel.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#import "MyView.h"
#import "MyLine.h"

static NSPoint scalePointInRect(NSPoint pt, NSRect rect)
{
    NSPoint scaledPt = NSMakePoint(rect.origin.x + pt.x * rect.size.width, 
                                   rect.origin.y + pt.y * rect.size.height);
    return scaledPt;
}

// private methods
@interface MyView(Private)
- (void)resetStats;
- (void)updateAvgDrawingRate;
- (void)setAvgDrawingRate:(float)rate;
- (void)updateAvgFps;
- (void)setAvgFps:(float)aVal;
@end

@implementation MyView

- (id)initWithFrame:(NSRect)frame
{
    self = [super initWithFrame:frame];
    if (self == nil)
        return nil;
    
    lines = nil;
    numLines = 0;
    useLineColor = YES;
    useLineWidth = YES;
    useAntiAliasing = YES;
    [self resetStats];
    return self;
}

- (void)resetStats
{
    timeSpentDrawing = 0.0;
    numRedraws = 0;
    initTime = [NSDate timeIntervalSinceReferenceDate];
}

// the only reason we need the following 'set' accessors is so we can resetStats
// Otherwise there is no need for them as bindings will do everything
- (void)setUseLineColor:(BOOL)state
{
    useLineColor = state;
    [self resetStats];
}

- (void)setUseLineWidth:(BOOL)state
{
    useLineWidth = state;
    [self resetStats];
}

- (void)setUseAntiAliasing:(BOOL)state
{
    useAntiAliasing = state;
    [self resetStats];
}

- (void)resizeWithOldSuperviewSize:(NSSize)oldBoundsSize
{
    // need to reset the stats when the view is resized
    //NSLog(@"resizeWithOldSuperviewSize");
    [self resetStats];
    [super resizeWithOldSuperviewSize: oldBoundsSize];
}

- (int)numRedraws
{
    return numRedraws;
}

- (float)avgDrawingRate  // average number of lines drawn per second
{
    return avgDrawingRate;
}

- (void)setAvgDrawingRate:(float)rate // called from updateAvgDrawingRate so that KVO works
{
    avgDrawingRate = rate;
}

- (void)updateAvgDrawingRate
{
    float rate = 0.0;
    if (timeSpentDrawing != 0.0)
    {
        rate = [lines count] * (numRedraws / timeSpentDrawing);
    }
    
    [self setAvgDrawingRate: rate];
}

- (float)avgFps // average number of view redraws per second
{
    return avgFps;
}

- (void)setAvgFps:(float)aVal // called from updateAvgFps so that KVO works
{
    avgFps = aVal;
}

- (void)updateAvgFps
{
    NSTimeInterval now = [NSDate timeIntervalSinceReferenceDate];
    float elapsedSec = now - initTime;
    [self setAvgFps: (numRedraws / elapsedSec)];
}

- (void)updateWithLines:(NSArray *)newLines
{
    if ([newLines count] != numLines)
    {
        //NSLog(@"resetting since numLines has changed");
        [self resetStats];
    }
    
    [newLines retain];
    [lines release];
    lines = newLines;
    numLines = [lines count];
    [self setNeedsDisplay: YES];
}

- (BOOL)isOpaque
{
    // this allows the graphics system to draw faster
    return YES;
}

- (void)drawRect:(NSRect)rect
{
    //NSLog(@"MyView.drawRect");
    NSRect bds = [self bounds];
    NSTimeInterval startTime = [NSDate timeIntervalSinceReferenceDate];
    
    NSGraphicsContext *gc = [NSGraphicsContext currentContext];
    [gc setShouldAntialias: useAntiAliasing];

    // clear the view to solid black background
    [[NSColor blackColor] set];
    NSRectFill(bds);
    
    if (useLineColor == NO)
    {
        // draw all lines in white
        [[NSColor whiteColor] set];
    }
    
    BOOL allLinesTheSame = (useLineColor == NO) && (useLineWidth == NO);
    if (allLinesTheSame)
    {
        NSBezierPath *path = [NSBezierPath bezierPath];
        for (int i = 0; i < numLines; i++)
        {
            MyLine *line = [lines objectAtIndex: i];
            NSPoint start = scalePointInRect([line start], bds);
            NSPoint end = scalePointInRect([line end], bds);
            [path moveToPoint: start];
            [path lineToPoint: end];
        }
        [path stroke];
    }
    else
    {
        for (int i = 0; i < numLines; i++) 
        {
            MyLine *line = [lines objectAtIndex: i];
            
            if (useLineColor == YES)
            {
                NSColor *colour = [line color];
                [colour set];
            }
            
            NSPoint start = scalePointInRect([line start], bds);
            NSPoint end = scalePointInRect([line end], bds);
            
            NSBezierPath *path = [NSBezierPath bezierPath];
            if (useLineWidth == YES)
            {
                float lineWidth = [line width];
                [path setLineWidth: lineWidth];
            }
            
            [path moveToPoint: start];
            [path lineToPoint: end];
            [path stroke];
        }
    }
    
    NSTimeInterval endTime = [NSDate timeIntervalSinceReferenceDate];
    float timeDrawing = (endTime - startTime);
    //NSLog(@"timeDrawing: %f", timeDrawing);
    timeSpentDrawing += timeDrawing;
    numRedraws++;
    [self updateAvgDrawingRate];
    [self updateAvgFps];
}

@end
